@@ -6,18 +6,23 @@
  This module also houses GEOS Pointer utilities, including
  get_pointer_arr(), and GEOM_PTR.
 """
+import logging
 import os
 import re
-import sys
 from ctypes import c_char_p, Structure, CDLL, CFUNCTYPE, POINTER
 from ctypes.util import find_library
+
 from django.contrib.gis.geos.error import GEOSException
+from django.core.exceptions import ImproperlyConfigured
+
+logger = logging.getLogger('django.contrib.gis')
 
 # Custom library path set?
 try:
     from django.conf import settings
     lib_path = settings.GEOS_LIBRARY_PATH
-except (AttributeError, EnvironmentError, ImportError):
+except (AttributeError, EnvironmentError,
+        ImportError, ImproperlyConfigured):
     lib_path = None
 
 # Setting the appropriate names for the GEOS-C library.
@@ -33,18 +38,21 @@ else:
     raise ImportError('Unsupported OS "%s"' % os.name)
 
 # Using the ctypes `find_library` utility to find the path to the GEOS
-# shared library.  This is better than manually specifiying each library name
+# shared library.  This is better than manually specifying each library name
 # and extension (e.g., libgeos_c.[so|so.1|dylib].).
 if lib_names:
     for lib_name in lib_names:
         lib_path = find_library(lib_name)
-        if not lib_path is None: break
+        if lib_path is not None:
+            break
 
 # No GEOS library could be found.
 if lib_path is None:
-    raise ImportError('Could not find the GEOS library (tried "%s"). '
-                        'Try setting GEOS_LIBRARY_PATH in your settings.' %
-                        '", "'.join(lib_names))
+    raise ImportError(
+        'Could not find the GEOS library (tried "%s"). '
+        'Try setting GEOS_LIBRARY_PATH in your settings.' %
+        '", "'.join(lib_names)
+    )
 
 # Getting the GEOS C library.  The C interface (CDLL) is used for
 # both *NIX and Windows.
@@ -56,36 +64,54 @@ lgeos = CDLL(lib_path)
 # Supposed to mimic the GEOS message handler (C below):
 #  typedef void (*GEOSMessageHandler)(const char *fmt, ...);
 NOTICEFUNC = CFUNCTYPE(None, c_char_p, c_char_p)
-def notice_h(fmt, lst, output_h=sys.stdout):
+
+
+def notice_h(fmt, lst):
+    fmt, lst = fmt.decode(), lst.decode()
     try:
         warn_msg = fmt % lst
-    except:
+    except TypeError:
         warn_msg = fmt
-    output_h.write('GEOS_NOTICE: %s\n' % warn_msg)
+    logger.warning('GEOS_NOTICE: %s\n' % warn_msg)
 notice_h = NOTICEFUNC(notice_h)
 
 ERRORFUNC = CFUNCTYPE(None, c_char_p, c_char_p)
-def error_h(fmt, lst, output_h=sys.stderr):
+
+
+def error_h(fmt, lst):
+    fmt, lst = fmt.decode(), lst.decode()
     try:
         err_msg = fmt % lst
-    except:
+    except TypeError:
         err_msg = fmt
-    output_h.write('GEOS_ERROR: %s\n' % err_msg)
+    logger.error('GEOS_ERROR: %s\n' % err_msg)
 error_h = ERRORFUNC(error_h)
 
 #### GEOS Geometry C data structures, and utility functions. ####
 
+
 # Opaque GEOS geometry structures, used for GEOM_PTR and CS_PTR
-class GEOSGeom_t(Structure): pass
-class GEOSPrepGeom_t(Structure): pass
-class GEOSCoordSeq_t(Structure): pass
-class GEOSContextHandle_t(Structure): pass
+class GEOSGeom_t(Structure):
+    pass
+
+
+class GEOSPrepGeom_t(Structure):
+    pass
+
+
+class GEOSCoordSeq_t(Structure):
+    pass
+
+
+class GEOSContextHandle_t(Structure):
+    pass
 
 # Pointers to opaque GEOS geometry structures.
 GEOM_PTR = POINTER(GEOSGeom_t)
 PREPGEOM_PTR = POINTER(GEOSPrepGeom_t)
 CS_PTR = POINTER(GEOSCoordSeq_t)
-CONTEXT_PTR  = POINTER(GEOSContextHandle_t)
+CONTEXT_PTR = POINTER(GEOSContextHandle_t)
+
 
 # Used specifically by the GEOSGeom_createPolygon and GEOSGeom_createCollection
 #  GEOS routines
@@ -95,7 +121,7 @@ def get_pointer_arr(n):
     return GeomArr()
 
 # Returns the string version of the GEOS library. Have to set the restype
-# explicitly to c_char_p to ensure compatibility accross 32 and 64-bit platforms.
+# explicitly to c_char_p to ensure compatibility across 32 and 64-bit platforms.
 geos_version = lgeos.GEOSversion
 geos_version.argtypes = None
 geos_version.restype = c_char_p
@@ -106,6 +132,8 @@ version_regex = re.compile(
     r'^(?P<version>(?P<major>\d+)\.(?P<minor>\d+)\.(?P<subminor>\d+))'
     r'((rc(?P<release_candidate>\d+))|dev)?-CAPI-(?P<capi_version>\d+\.\d+\.\d+)( r\d+)?$'
 )
+
+
 def geos_version_info():
     """
     Returns a dictionary containing the various version metadata parsed from
@@ -113,7 +141,7 @@ def geos_version_info():
     is a release candidate (and what number release candidate), and the C API
     version.
     """
-    ver = geos_version()
+    ver = geos_version().decode()
     m = version_regex.match(ver)
     if not m:
         raise GEOSException('Could not parse version info string "%s"' % ver)
@@ -127,22 +155,10 @@ GEOS_MINOR_VERSION = int(_verinfo['minor'])
 GEOS_SUBMINOR_VERSION = int(_verinfo['subminor'])
 del _verinfo
 GEOS_VERSION = (GEOS_MAJOR_VERSION, GEOS_MINOR_VERSION, GEOS_SUBMINOR_VERSION)
-GEOS_PREPARE = GEOS_VERSION >= (3, 1, 0)
 
-if GEOS_PREPARE:
-    # Here we set up the prototypes for the initGEOS_r and finishGEOS_r
-    # routines.  These functions aren't actually called until they are
-    # attached to a GEOS context handle -- this actually occurs in
-    # geos/prototypes/threadsafe.py.
-    lgeos.initGEOS_r.restype = CONTEXT_PTR
-    lgeos.finishGEOS_r.argtypes = [CONTEXT_PTR]
-else:
-    # When thread-safety isn't available, the initGEOS routine must be called
-    # first.  This function takes the notice and error functions, defined
-    # as Python callbacks above, as parameters. Here is the C code that is
-    # wrapped:
-    #  extern void GEOS_DLL initGEOS(GEOSMessageHandler notice_function, GEOSMessageHandler error_function);
-    lgeos.initGEOS(notice_h, error_h)
-    # Calling finishGEOS() upon exit of the interpreter.
-    import atexit
-    atexit.register(lgeos.finishGEOS)
+# Here we set up the prototypes for the initGEOS_r and finishGEOS_r
+# routines.  These functions aren't actually called until they are
+# attached to a GEOS context handle -- this actually occurs in
+# geos/prototypes/threadsafe.py.
+lgeos.initGEOS_r.restype = CONTEXT_PTR
+lgeos.finishGEOS_r.argtypes = [CONTEXT_PTR]
