@@ -1,13 +1,16 @@
-from __future__ import absolute_import, unicode_literals
+from __future__ import unicode_literals
 
 from django import forms
 from django.forms.formsets import BaseFormSet, DELETION_FIELD_NAME
-from django.forms.util import ErrorDict, ErrorList
+from django.forms.utils import ErrorDict, ErrorList
 from django.forms.models import modelform_factory, inlineformset_factory, modelformset_factory, BaseModelFormSet
 from django.test import TestCase
 from django.utils import six
 
-from .models import User, UserSite, Restaurant, Manager, Network, Host
+from .models import (
+    User, UserSite, UserProfile, ProfileNetwork, Restaurant, Manager, Network,
+    Host,
+)
 
 
 class InlineFormsetTests(TestCase):
@@ -154,6 +157,37 @@ class InlineFormsetTests(TestCase):
         else:
             self.fail('Errors found on formset:%s' % form_set.errors)
 
+    def test_inline_model_with_to_field(self):
+        """
+        #13794 --- An inline model with a to_field of a formset with instance
+        has working relations.
+        """
+        FormSet = inlineformset_factory(User, UserSite, exclude=('is_superuser',))
+
+        user = User.objects.create(username="guido", serial=1337)
+        UserSite.objects.create(user=user, data=10)
+        formset = FormSet(instance=user)
+
+        # Testing the inline model's relation
+        self.assertEqual(formset[0].instance.user_id, "guido")
+
+    def test_inline_model_with_to_field_to_rel(self):
+        """
+        #13794 --- An inline model with a to_field to a related field of a
+        formset with instance has working relations.
+        """
+        FormSet = inlineformset_factory(UserProfile, ProfileNetwork, exclude=[])
+
+        user = User.objects.create(username="guido", serial=1337, pk=1)
+        self.assertEqual(user.pk, 1)
+        profile = UserProfile.objects.create(user=user, about="about", pk=2)
+        self.assertEqual(profile.pk, 2)
+        ProfileNetwork.objects.create(profile=profile, network=10, identifier=10)
+        formset = FormSet(instance=profile)
+
+        # Testing the inline model's relation
+        self.assertEqual(formset[0].instance.profile_id, 1)
+
     def test_formset_with_none_instance(self):
         "A formset with instance=None can be created. Regression for #11872"
         Form = modelform_factory(User, fields="__all__")
@@ -161,8 +195,8 @@ class InlineFormsetTests(TestCase):
 
         # Instantiate the Form and FormSet to prove
         # you can create a formset with an instance of None
-        form = Form(instance=None)
-        formset = FormSet(instance=None)
+        Form(instance=None)
+        FormSet(instance=None)
 
     def test_empty_fields_on_modelformset(self):
         "No fields passed to modelformset_factory should result in no fields on returned forms except for the id. See #14119."
@@ -203,7 +237,7 @@ class InlineFormsetTests(TestCase):
         self.assertQuerysetEqual(
             dalnet.host_set.order_by("hostname"),
             ["<Host: matrix.de.eu.dal.net>", "<Host: tranquility.hub.dal.net>"]
-            )
+        )
 
     def test_initial_data(self):
         user = User.objects.create(username="bibi", serial=1)
@@ -330,6 +364,7 @@ class FormfieldCallbackTests(TestCase):
                              formfield_callback=callback)
         self.assertCallbackCalled(callback)
 
+
 class BaseCustomDeleteFormSet(BaseFormSet):
     """
     A formset mix-in that lets a form decide if it's to be deleted.
@@ -369,25 +404,25 @@ class FormfieldShouldDeleteFormTests(TestCase):
     DeleteFormset = modelformset_factory(User, form=CustomDeleteUserForm, formset=BaseCustomDeleteModelFormSet)
 
     data = {
-            'form-TOTAL_FORMS': '4',
-            'form-INITIAL_FORMS': '0',
-            'form-MAX_NUM_FORMS': '4',
-            'form-0-username': 'John',
-            'form-0-serial': '1',
-            'form-1-username': 'Paul',
-            'form-1-serial': '2',
-            'form-2-username': 'George',
-            'form-2-serial': '3',
-            'form-3-username': 'Ringo',
-            'form-3-serial': '5',
-            }
+        'form-TOTAL_FORMS': '4',
+        'form-INITIAL_FORMS': '0',
+        'form-MAX_NUM_FORMS': '4',
+        'form-0-username': 'John',
+        'form-0-serial': '1',
+        'form-1-username': 'Paul',
+        'form-1-serial': '2',
+        'form-2-username': 'George',
+        'form-2-serial': '3',
+        'form-3-username': 'Ringo',
+        'form-3-serial': '5',
+    }
 
     delete_all_ids = {
-            'form-0-DELETE': '1',
-            'form-1-DELETE': '1',
-            'form-2-DELETE': '1',
-            'form-3-DELETE': '1',
-            }
+        'form-0-DELETE': '1',
+        'form-1-DELETE': '1',
+        'form-2-DELETE': '1',
+        'form-3-DELETE': '1',
+    }
 
     def test_init_database(self):
         """ Add test data to database via formset """
@@ -405,7 +440,7 @@ class FormfieldShouldDeleteFormTests(TestCase):
         data['form-INITIAL_FORMS'] = 4
         data.update(dict(
             ('form-%d-id' % i, user.pk)
-            for i,user in enumerate(User.objects.all())
+            for i, user in enumerate(User.objects.all())
         ))
         formset = self.NormalFormset(data, queryset=User.objects.all())
         self.assertTrue(formset.is_valid())
@@ -422,7 +457,7 @@ class FormfieldShouldDeleteFormTests(TestCase):
         data['form-INITIAL_FORMS'] = 4
         data.update(dict(
             ('form-%d-id' % i, user.pk)
-            for i,user in enumerate(User.objects.all())
+            for i, user in enumerate(User.objects.all())
         ))
         data.update(self.delete_all_ids)
         formset = self.NormalFormset(data, queryset=User.objects.all())
@@ -441,7 +476,7 @@ class FormfieldShouldDeleteFormTests(TestCase):
         data['form-INITIAL_FORMS'] = 4
         data.update(dict(
             ('form-%d-id' % i, user.pk)
-            for i,user in enumerate(User.objects.all())
+            for i, user in enumerate(User.objects.all())
         ))
         data.update(self.delete_all_ids)
         formset = self.DeleteFormset(data, queryset=User.objects.all())
@@ -454,3 +489,52 @@ class FormfieldShouldDeleteFormTests(TestCase):
         # verify no "odd" PKs left
         odd_ids = [user.pk for user in User.objects.all() if user.pk % 2]
         self.assertEqual(len(odd_ids), 0)
+
+
+class RedeleteTests(TestCase):
+    def test_resubmit(self):
+        u = User.objects.create(username='foo', serial=1)
+        us = UserSite.objects.create(user=u, data=7)
+        formset_cls = inlineformset_factory(User, UserSite, fields="__all__")
+        data = {
+            'serial': '1',
+            'username': 'foo',
+            'usersite_set-TOTAL_FORMS': '1',
+            'usersite_set-INITIAL_FORMS': '1',
+            'usersite_set-MAX_NUM_FORMS': '1',
+            'usersite_set-0-id': six.text_type(us.pk),
+            'usersite_set-0-data': '7',
+            'usersite_set-0-user': 'foo',
+            'usersite_set-0-DELETE': '1'
+        }
+        formset = formset_cls(data, instance=u)
+        self.assertTrue(formset.is_valid())
+        formset.save()
+        self.assertEqual(UserSite.objects.count(), 0)
+        formset = formset_cls(data, instance=u)
+        # Even if the "us" object isn't in the DB any more, the form
+        # validates.
+        self.assertTrue(formset.is_valid())
+        formset.save()
+        self.assertEqual(UserSite.objects.count(), 0)
+
+    def test_delete_already_deleted(self):
+        u = User.objects.create(username='foo', serial=1)
+        us = UserSite.objects.create(user=u, data=7)
+        formset_cls = inlineformset_factory(User, UserSite, fields="__all__")
+        data = {
+            'serial': '1',
+            'username': 'foo',
+            'usersite_set-TOTAL_FORMS': '1',
+            'usersite_set-INITIAL_FORMS': '1',
+            'usersite_set-MAX_NUM_FORMS': '1',
+            'usersite_set-0-id': six.text_type(us.pk),
+            'usersite_set-0-data': '7',
+            'usersite_set-0-user': 'foo',
+            'usersite_set-0-DELETE': '1'
+        }
+        formset = formset_cls(data, instance=u)
+        us.delete()
+        self.assertTrue(formset.is_valid())
+        formset.save()
+        self.assertEqual(UserSite.objects.count(), 0)
